# Changelog
All notable changes of the WooCommerce Bexio Automator are listed here.

## 2.4.1 (2024-03-27)

### Fixes and improvements
* Fix possible SQL error when trying to create a new stock cache table if it does not yet exist
* Adjust migration for existing stock cache tables (since version 2.4.0) to use the same default value for the cache time as a newly created table


## 2.4.0 (2024-03-19)

### Fixes and improvements
* Refactoring of stock cache handling and stock sync
* Fixed bug where, when a bexio product with the given SKU no longer exists, manually syncing stock could lead to an old cached stock value being applied to the WC product

#### Changes to the stock cache handling
The refactoring includes the following changes:
* Products with empty SKU or disabled stock-management will be more consequently ignored in the stock cache and sync (also in the async stock synchronization)
* The cache time is now stored directly in the stock cache table (instead of as postmeta value) for better querrying performance
* Migrate existing stock cache times from the postmeta table to the stock cache table
* Changed lookup querries to use the wc_product_meta_lookup instead of the postmeta table for better performance
* Updated the handling of when the stock cache time is set/updated (is now set directly when writing stocks to the cache table)
* Remove entries with empty SKU or expired cache time (including 6h grace period) after writing new stocks to the cache
* Simplified checks for expired cache time
* Added new methods to check and update cache time via either the SKU or product-id (as value is no longer stored in postmeta table)
* When using the manual stock sync, compare received stocks to requested SKUs and remove entries for which no update was received (because no Bexio product exists)
* Updated last sync column in the product dashboard to the correct value in the cache table
* After checkout validation, when current stock in Bexio is sufficient, set an expired cache time to ensure that *after* the completed checkout, the new stock (excluding ordered quantity) gets cached when next viewing said stock

### Other changes
* Add new plugin header 'Requires Plugins' with corresponding dependency in preparation for WP 6.5


## 2.3.0 (2024-02-26)

### New Features
* Add new settings field to allow selecting order comments to be synchronized as text positions


## 2.2.3 (2024-02-07)

### Fixes and improvements
* Fix potential problem, where having old product stock in the cache table could lead to WC stock being overwrtitten with an old value, if the originating Bexio product for said stock was since removed and said stock is thus no longer being updated


## 2.2.2 (2024-02-02)

### Fixes and improvements
* Fix check for missing skus when using bulk action to sync stock of specific products


## 2.2.1 (2024-02-01)

### Fixes and improvements
* Fix potential error in checkout stock validation when attempting to get SKU but product is empty


## 2.2.0 (2024-01-31)

### New Features
* Add a notification system to allow public notifications for all plugin users to be shown as admin notices

### Fixes and improvements
* Fix bug where syncing stock of a specific variable product, which has no SKUs, can lead to stock of its variations being overwritten

### Other changes
* Rename config variable for troubleshooting URL
* Remove config variables for subscription prices (see [website](https://www.bexio-shop.ch/pricing) for prices)
* Adjust mail sent for failed order sync (encourage usage of the FAQ [troubleshooting article](https://helpcenter.bexio-shop.ch/hc/de/articles/9607181111569))
* Change sender address of mails sent for failed order sync to WordPress admin email to better avoid possible spam classification
* Change FAQ URL in mail sent for failed PDF download to the [troubleshooting article](https://helpcenter.bexio-shop.ch/hc/de/articles/9607181111569)
* Bump tested WordPress and WooCommerce versions in plugin information
* Adjust link to FAQ in config - point to correct [helpcenter section](https://helpcenter.bexio-shop.ch/hc/de/sections/9576363732369) instead of the previous redirected generalized URL
* Change help-link in the plugin dashboard to the FAQ
* Change link in synchronization rules description to point directly to the [specific FAQ article](https://helpcenter.bexio-shop.ch/hc/de/articles/9606928850961) (instead of old URL redirected to the general FAQ)
* Adjust stock validation during checkout to skip syncing stock from Bexio for products that have stock management enabled, but do not have an SKU


## 2.1.8 (2024-01-22)

### New Features
* Add checkbox to enable or disable rounding of prices of orders sent to Bexio


## 2.1.7 (2024-01-15)

### Fixes and improvements
* Add new hooks to allow filtering the order query for tasks
* Bug: If manage_stock is disabled in WooCommerce and Bexio stock is 0, the available product should remain available


## 2.1.6 (2023-12-18)

### Fixes and improvements
* Adjust calculation of discount per position to explicitly cast to float value and use more generic value comparison to prevent division errors for users who might have ended up with string values for the regular price


## 2.1.5 (2023-12-14)

### Fixes and improvements
* Adjust abort condition when trying to calculate discount per position (abort on either 0 or 0.0)


## 2.1.4 (2023-12-12)

### Fixes and improvements
* Improve stock table creation and add check of result in mysqli_fetch_object


## 2.1.3 (2023-11-14)

### Fixes and improvements
* Fix cron job task for async stock synchronization, so the hourly cron job for the stock is compatible with WordPress v6.4


## 2.1.2 (2023-11-08)

### Fixes and improvements
* Fix error with fallback to old `mysql_query`, resulting from changes with database host after update to WordPress v6.4, since they removed conditional code and methods to check usage of mysqli (mysqli is _expected_ now), thus triggering the fallback usage of `mysql_query` instead of `mysqli_query`


## 2.1.1 (2023-11-08)

### Fixes and improvements
* Fix possible 'undefined array key' warning when checking if a language is already set in the request data


## 2.1.0 (2023-10-31)

### New Features
* Add plugin filters and actions documentation in readme file
* Add new functionality to 'async' sync mode of orders: 'new' orders (only those not yet synced at all) are synced to bexio every 5 minutes; the previous async task (run 1 / hour) will still run regardless (for additional sync where necessary)
* Add filter to optionally disable seperate (every 5 minutes) sync for 'new' orders (with 'async' sync mode)

### Fixes and improvements
* Adjust 'Accept-Language' header when requesting data to prevent incorrect string values as that might result in incorrect translation data from Bexio
* Switch to wp_json_encode (instead of json_encode) to encode data before synchronization (for additional sanitization)
* Adjust search for existing product (SKU) in product import module: do not narrow search for SKU to specific post_types (search in all types) and use wpdb instead of get_posts() to avoid non-public product post-types causing the post-type to possibly be ignored in get_posts()
* Add admin-notice after product import to show result of how many products were created or updated
* Add try/catch block to import of a product from bexio, count caught exceptions to list number of products failed to import because they errored out and continue with the rest of the import after catching an error

### Other changes
* Make reset of plugin-data harder by removing the reset button, to prevent customers 'just trying this real quick' (and causing further problems by thus deleting the connection to the existing subscription) - the reset of the authorization with Bexio remains unaffected
* Remove trial subscription and initial setup package options from the manage account page, to prevent requests for a trial subscription, although a full trial has already expired, without any other comments or a request for a setup package (i.e. requesting no valid changes)
* Add comment description to method in tasks class
* Adjust subscription management page - remove subscription prices (refer to website) and adjust text for setup package (is already referred to website)
* Adjust account management page - only show expiration date for trials or cancelled accounts and adjust texts accordingly; adjust text in cancellation form (do not display expiration date)
* Show account number instead of expiration date in account status to help identify the correct account for support cases
* Do not show account-expired-message in settings when subscription type is not trial
* Show a different message for blocked accounts that are expired but are not trials


## 2.0.3 (2023-09-19)

### New Features
* Add new filter to allow forcing activation of tasks

### Other changes
* Init plugin tasks on a later hook to allow filtering from the theme
* Rename filter in invoicestate sync task to differentiate from orderstate sync task


## 2.0.2 (2023-09-14)

### Fixes and improvements
* Fix modules not being available when authorization is reset but wizard was completed, leading to settings of additional modules being lost in sanitization when updating options via code


## 2.0.1 (2023-09-05)

### New Features
* Show notice in backend when migration to new plugin version requires reauthorization with bexio

### Other changes
* Add new parameter in utils method to skip logging for backend notices


## 2.0.0 (2023-08-30)

### New Features
* Use new version of plugin-service (uses Bexio API v3)
* Enable support for document template selection (now works with Bexio API v3)
* Adjust display of tax selection (Bexio API v3 now provides a clearer display-name)
* Adjust file cache - add a htaccess file as fallback

### Fixes and improvements
* Add fixes for changes from switching to Bexio API v3
* Add notice regarding potential problems if not using the primary Bexio user (ID 1) when authorizing
* Optimize country-settings module to prevent potential timeout issues when using 'ship to all countries'
* Adjust FAQ url in email for failed sync (directly link to article for troubleshooting sync errors)
* Fix incorrect (or completely skipped) discount calculation (for discount-sync module) when total of product is 0.- (e.g. when 100% discount should be applied)
* Refactor abort condition when trying to calculate discount per position (to better catch null, false or '')
* Adjust file cache - check if function chmod exists before usage
* Fix potential redirect loop when getting a new authorize url failed

### Other changes
* Adjust for new authorization code flow
* Adjust fields for Bexio API v3
* Add Accept-Language header to requests from the plugin (setting up for potential feature later)
* Added settings migration functionality
* Add migration for plugin version 2.0.0 - remove Bexio authorization (re-authorizing is mandatory) and clear file-cache


## 1.17.1 (2023-06-28)

### New Features
* Add new filter to allow overriding the payment gateway id to check synchronization rules for


## 1.17.0 (2023-06-26)

### New Features
* Add new option to set behaviour for additional addresses on shipping address
* Replace plugin branding
* Add button on invalid salt blocked message to request resend of verification email

### Fixes and improvements
* Add check for wizard=done in api and settings classes to try to eliminate potential configuration loss
* Add error message if json_encode fails and do not send a request in that case
* Fix invalid salt blocked message maybe not being visible

### Other changes
* Adjust automatic redirect after authorization
* Remove automatic redirect on invalid salt mail resend (manual refresh more suited)
* Remove unreachable code


## 1.16.0 (2023-05-05)

### New Features
* Add options to select a reason when requesting cancellation of the account
* Add warning notice regarding potential synchronization errors caused by wrong booking accounts or tax settings before continuing with the setup wizard

### Fixes and improvements
* Update `wcs_{type}_meta_query` filters to `wc_subscriptions_{type}_data` filters for woocommerce-subscriptions compatibility
* Fix warning with missing 'page' GET parameter on other pages
* Optimize enqueueing of wizard and admin styles
* Update the verification key when a new one is generated in the service
* Show a different message when the account is blocked because of a divergent verification key
* Fix account blocking-cause potentially being empty and thus not showing certain messages
* Add method to clear a specific cache file & automatically clear client status when refreshing from a blocked account
* Adjust cache file filemod from 0600 to 0700 to allow automatic deletion of client status cache when checking verification


## 1.15.2 (2023-04-21)

### Fixes and improvements
* Prevent potential error when using 'screen_layout_columns' filter if other hooks set non-array value
* Change filemods for cache files to 0600 to prevent non-owners reading the cache files


## 1.15.1 (2023-03-28)

### Fixes and improvements
* Fix initial logging of current configuration blocking setup during new installation


## 1.15.0 (2023-03-27)

### New Features
* Add remote logging for errors, configuration changes, module changes and authorization / data resets
* Add functionality to send initial plugin configuration to remote storage

### Fixes and improvements
* Add additional check for update information


## 1.14.2 (2023-03-16)

### Fixes and improvements
* Fix WooCommerce payment gateways potentially not showing up in settings, depending on load order, when Bexio Automator reads the gateways from WooCommerce for settings / defaults


## 1.14.1 (2023-02-21)

### Fixes and improvements
* Show error notice if WP_HTTP_BLOCK_EXTERNAL ist true and WP_ACCESSIBLE_HOSTS is set to foreign API
* Register function migrate_stock at init hook RSD-10788
* NULL check for values in discountsync module BS-1963
* Adds check if regular price or total are null or empty strings to stop discount calculation
* Add alternative method of getting product information for order_item_data in order to determine regular price for discount sync (to work for manual orders too)
* Replace hook and check if item is real product
* Refactor abort conditions in discountsync module (make it simpler and more readable)
* Fix 'Creation of dynamic property ...  is deprecated' warning
* Adds check if plugin information for update is available


## 1.14.0 (2023-02-01)

### New Features
* Add new discountsync module, allowing to sync discounts per position instead of as separate discount position
* Add translations for new module & settings

### Fixes and improvements
* Fix modules being unable to read their own new options with correct default if other modules / classes already read options and thus cause defaults filter to already be applied (RSD-10758)
* Hide discount module order item meta data in backend and fix discount calculation potentially failing because regular price needed for difference calculation being falsified by B2B functionality (BS-1830)
* Fix position price when using discount per position (as bexio will subtract discount automatically, potentially leading to double subtraction of discount if regular price is not set explicitly)

### Other changes
* Add new action to enable adding settings in between existing settings
* Add new filter allowing to remove existing settings fields
* Refactor existing module settings for new register_setting filter and after_settings action


## 1.13.2 (2023-01-20)

### Fixes and improvements
* Prevent stock refresh for products for which the cache is not yet expired
* Set sslverify to 1
* Add compatibility for GTranslate Plugin
* Adjust sslverify to adapt depending on WP_DEBUG to prevent cURL error 60 on dev environments
* change defined check for classes directory constant CLASSES_DIR


## 1.13.2-b4 (2022-09-15)

### Fixes and improvements
* RSD-10424 Adds check for Extended Sync Option to show Woocommerce Payments - removes old_options


## 1.13.2-b3 (2022-09-14)

### Fixes and improvements
* BS-1820 adds check if status is not false in order to suppress warning: Trying to access array offset on value of type bool


## 1.13.2-b2

### Fixes and improvements
* BS-1812 Bugfix: add slashes to comply with preg_match in class.options.php


## 1.13.2-b1

### Fixes and improvements
* Adds 3 irreversible endpoint states for invoice and order sync tasks


## 1.13.1 (2022-07-25)

### Fixes and improvements
* Separate Connection Options from Regular Options
* Fix Registration of Plugin Cron Job of Invoice Sync


## 1.13.0 (2022-07-01)

### New Features
* Add new module for extended state synchronization (allow synchronizing by invoice state)

### Fixes and improvements
* Clean up code - remove unused methods & left over commented code, add order-state-transition check and re-activate order-state-transition filter for invoice sync task, correct method descriptions and general gardening


## 1.12.2 (2022-05-10)

### Fixes and improvements
* Add functionality to convert size & weigt to the units selected in WooCommerce when importing products from Bexio
* Fix deactivate method on modules not being called when module is not enabled
* Fix price sync task continuing to execute when productimport module is deactivated after task was enabled
* Fix stock update bulk action not updating stock of product variations


## 1.12.1 (2022-04-22)

### Fixes and improvements
* Add compatibility with Woocommerce 6.4.1. Removed insertion of meta-information for variable product positions


## 1.12.0 (2022-04-19)

### New Features
* Added new module to allow setting country specific booking account and tax rates and added new filters to implement new functionality

### Fixes and improvements
* Fix defaults being overridden during setup wizard, fix sync rules defaults not being applied correctly and update defaults for booking accounts to new standard bexio id's


## 1.11.2-b1 (2022-03-29)

### Fixes and improvements
* Add check for successful stock table migration (BS-1631)
* Force sync when using bulk action
* Fix linebreaks in product excerpt not showing up correctly in Bexio


## 1.11.1 (2022-03-14)

### Fixes and improvements
* Prevent empty price in Bexio product unsetting existing price in WooCommerce


## 1.11.0 (2022-02-11)

### New Features
* Add new feature to allow automatic update of product price from the Bexio product (BS-1504)


## 1.10.6 (2022-02-09)

### Fixes and improvements
* Fix product specific fetch modes causing no stock to be synced for those products, fix stock sync bulk action causing sync of all stocks instead of only selected stocks


## 1.10.5 (2022-01-17)

### New Features
* Add new link to the downloaded bexio invoice pdf in the order details
* Add new email notification when download of bexio invoice pdf fails

### Fixes and improvements
* Adjust translation of documentnumbers module to exclude deliveries as those settings were removed because bexio does not support it for that type
* Major improvements to stock update handling to prevent issues when having a lot of products
* Only update stock status if changed
* Make sure PDF invoice is downloaded also if not sent by e-mail
* Correct typo in german translations
* Only auto download PDFs if status is not cancelled, status is not pending and status is not completed or we are on the order detail
* Update post meta storing the Invoice PDF URL rather than add in order to prevent duplicate entries

### Other changes
* Add new filters to allow changing the uploads subdirectory and filename under which the pdf of the bexio invoice is to be saved


## 1.10.4 (2021-08-11)

### Hotfixes
* Try to avoid an exception in certain circumstances with WC Subscriptions


## 1.10.3 (2021-07-19)

### Hotfixes
* Return unchanged stock amount when filtering stock quantity and no SKU is set

### Fixes and improvements
* Move order_item_data filter directly in front of array_push to allow overriding meta and excerpt text if needed


## 1.10.2 (2021-06-01)

### Hotfixes
* Allow syncing free orders without a payment method


## 1.10.1 (2021-05-27)

### Hotfixes
* Fix boolean return from `wc_get_product()` causing errors in stock-sync


## 1.10.0 (2021-05-11)

### New Features
* Add new module to allow importing / updating products from bexio `PRO`

### Fixes and improvements
* Disable support for custom document numbers for deliveries as the bexio API currently does not allow setting the document number when creating deliveries


## 1.9.4 (2021-05-05)

### New Features
* Add possibility to auto optimize address data (new setting)

### Fixes and improvements
* Fix a fatal error when a product is not available
* Adjust the function to fetch the stocks information of all products in order to improve performance
* Adjust the German translation of 'Coupon' text on discount positions as the ':' at the end is misleading
* Introduce a new filter `raptus_wc_bexio_connector_update_stock` to allow setting custom stock status
* Implement a file based cache to avoid blocking the backend during Bexio timeouts and increase performance

### Other changes
* Add a new filter when transitioning order states to improve WC Subscriptions compatibility
* Use WooCommerce 3 CRUD methods if available to set the stock status
* Remove some version compatibility checks, as they are already given through the use of native WooCommerce stock functions
* Add number of skipped products to admin notice


## 1.9.3 (2021-01-15)

### Hotfixes
* Prevent orders without a payment method and containing only virtual products from being synched


## 1.9.2 (2021-01-15)

### Hotfixes
* Do not copy bexio meta data to subscription and renewal orders (woocommerce subscriptions compatibility)


## 1.9.1 (2020-12-22)

### Hotfixes
* Prevent a fatal error on checkout when ordering gifts using the [Free Gifts for WooCommerce](https://woocommerce.com/products/free-gifts-for-woocommerce/) plugin

### New features
* Add new function to manually unlink a WooCommerce order from the bexio documents


## 1.9.0 (2020-12-10)

### New features
* Add a guided wizard for the initial installation
* Implement asynchronous stock synchronization `PRO`
* Add possibility to define stock fetch modes on product and product categories `PRO`
* Add new module to set extended stock synchronization options `PRO`
* Add new setting for asynchronous order syncing
* Implement new asynchronous order synchronization task
* Add new module for adding invoice PDF attachments to WooCommerce emails `PRO`
* Add new option to set a booking account for discounts
* Create a standard position in the bexio order if a booking account for discounts is set
* Implement new module for setting Bexio document numbers `PRO`
* Add tax group not_taxable_turnover to GET request to allow selecting UNO tax in tax settings

### Fixes and improvements
* Improve display of the currently active modules
* Improve the error logging
* Reload settings after activating modules
* Improve message type of partially synchronized orders
* Improve system for success and error messages for manual synchronization
* Improve the VAT tax rate settings layout and behavior

### Other changes
* Adjust sync result message and adjust discount logic
* Update and improve German and French translations
* Make modules implement the activatable interface
* Add debug function to validate bexio order data
* Plugin tested with WooCommerce version 4.6.0
* Extract the product booking accounts as separate module
* Implement plugin modularization


## 1.8.5 (2020-06-02)

### Hotfixes
* Fix error with unassociated bexio products during synchronization


## 1.8.4 (2020-04-27)

### Hotfixes
* Add more security checks to avoid errors if no product is associated with an order position


## 1.8.3 (2020-04-25)

### Hotfixes
* Fix error with recurring disconnection
* Fix fatal error if no product is associated with an order position

### Other changes
* Add missing support for products on backorder when synchronizing the stock
* Round prices based on WooCommerce settings


## 1.8.2 (2020-04-20)

### Hotfixes
* Catch a fatal error when transitioning the order status


## 1.8.1 (2020-03-30)

### Hotfixes
* Respect the bank account ID for product variations
* Show all enabled payment gateways


## 1.8.0 (2020-03-05)

### New features
* Implement flexible [synchronization rules configuration](https://bexio-shop.ch/faq/regelwerk/)
* Monitor all order status transitions and creation events
* Implement auto issuing of deliveries for completed orders
* Add synchroniatzion rules for virtual orders
* Implement new settings and compatibility for free order synchronization
* Add basic [WooCommerce Subscriptions](https://woocommerce.com/products/woocommerce-subscriptions/) plugin compatibility
* New meta key placeholder for the order title using syntax `{meta:key}`
* New option to override the invoice additional payment terms text
* Allow separate manual synchronization of orders, invoices, deliveries and payments

### Fixes and improvements
* Avoid a fatal error if WooCommerce is not available and print a notice
* Fix a fatal error when setting the invoice status
* Fix an error if an order no longer exists when synchronizing remote order states
* Fix a warning if the account is blocked and unauthorized
* Fix PHP warning when using a country code in the order title
* Avoid warning while checking for new plugin versions
* Fix order date format during synchronization

### Other changes
* Correctly handle the new cancelled client status
* Add new company field to the registration form
* Group tax rates based on their type for a easier configuration
* Allow net tax rates for the settings page
* Update all translations (English, German, French)
* Synchronize the order billing birthdate if possible
* Allow overriding the invoice due date for online payments


## 1.7.0 (2019-09-19)

### New features
* Rename the plugin to __WooCommerce Bexio Automator__
* Add a new option to disable the taxes via settings
* Allow synchronization of free shipping via a new settings option
* Introduce French translation of the entire plugin
* Add working default values for settings to facilitate the initial setup
* Add new bulk action for product stock synchronization with error handler
* Trigger error notification if the order synchronization fails
* Add new option to set error notification email and update translations
* Add new order synchronization failed email template
* Synchronize orders upon status transitions using a status monitor
* Deactivate WP cron jobs upon option or plugin deactivation
* Synchronize the order language when creating a new order via Bexio
* Add new placeholder for the payment method title of an order

### Fixes and improvements
* Implement a mechanism to avoid double synchronization of orders
* Make sure that boolean settings can be disabled
* Display all enabled payment methods, not only the available ones
* Correct double click prevention on order synchronize button; in some setups it completely disabled the function for manual synchronization
* Support autoloading when using uppercase path names
* Ignore remote order status `processing` to avoid marking unpaid invoices as paid
* Improve the error message display after manual synchronization
* Fix a payment method title problem in the order title
* Ensure WC2+ compatibility when fetching the payment date and fix several deprecation warnings

### Other changes
* Add new filter to allow overriding the request API URL
* Introduce a new filter to control if an invoice is synchronized and paid
* Remove unneeded error logging and debug statements
* Simplify the caching of API values using a magic method
* Eliminate some annoying PHP warnings
* Do not synchronize orders after creation
* Introduce sync status codes for more flexible displaying of messages
* Remove old check for field data that has been removed
* Increase cURL connection timeout to 45 seconds
* Add new API routes to pay already synchronized orders
* Add new API functions for payments and add API doc URL references
* Fix php doc errors and improve code formatting
* Add new filter to enable/disable stock synchronization
* Updated compatible WooCommerce version to 3.5.0
* Change informational text about the old 30 day trial period to 90 days
* Add a new utility function to request the language by post ID
* Get payment method title upon order synchronization
* Updated German translations of new notice messages
* Remove outdated dependency to WC_Product
* Add synchronize all action and fix several issues


## 1.6.0 (2018-10-15)

* Update the German translations for all new fields
* Add direct link to the bexio order in order detail and improve phpdoc
* Fix two minor bugs to improve speed
* Add days remaining message for trial accounts
* Add error message if user denies access to Bexio
* Implement new manage account actions and switch settings to the new data layer
* Add new utility function to fetch the current domain and improve the support error message
* Add new configuration constants for the bexio shop website
* Remove outdated code and clean up formatting
* Improve styling of some settings components
* Add new template for managing or cancelling the client subscription
* Add new template for displaying the current client status
* Add new utility class for sending emails to the WooCommerce Bexio Automator Support
* Create new data layer to reduce service requests and increase the speed
* Remove error log in API class
* Synchronize WooCommerce and WordPress verions for better support
* Synchronize to new contaction creation mode setting to the service
* Add new setting to specify the contact creation mode: flat or nested
* Add new cron task to synchronize order states between Bexio and WooCommerce
* Add new settings field to enable/disable order state synchronization
* Add a new API method to fetch a list of order states from the service
* Load the new product module upon initialization in the admin
* Synchronize the new bexio account settings to the service for each product position of the order
* Add new settings fields on products and product categories to select a bexio revenue account
* Fix incorrectly formatted changelog
* Load the new product module upon initialization in the admin
* Synchronize the new bexio account settings to the service for each product position of the order
* Add new settings fields on products and product categories to select a bexio revenue account
* Add formal German translation file
* Set Bexio bank account id for an order if configured
* Add new settings matrix to configure bank accounts for payment methods
* Add API method to fetch a list of Bexio bank accounts
* Translate new settings and error messages
* Remove unnecessary error logs
* Sync invoice payment if applicable and process response accordingly
* Add new settings for automatic invoice payment
* Fix title placeholders and replacements
* Make use of new admin notice utility function and display invoice related errors
* Add new utility function to display dismissable admin notices
* Activate order invoice creation and issuing based on payment status
* Add new settings to create and issue an invoice after the order creation
* Add new option to enable/disable appending the state after the city
* Add top product placeholder and replacement
* Introduce order title placeholder filter and allow nested structure
* Centralize all admin styles
* Move generic array flatten function to utils
* Improve stock update check
* Final fix for invalid refresh token issue


## 1.5.3 (2018-08-13)

* Final fix for invalid refresh token issue


## 1.5.2 (2018-08-13)

* Correctly parse bexio error if refresh token is invalid


## 1.5.1 (2018-08-10)

* Improve OAuth and Subscription error handling


## 1.5.0 (2018-08-08)

* Add admin notice about trial expiration
* Handle e-mail verification for unauthorized client update
* Improve onboarding process, allow data reset, handle failed authorization by allowing reauthorization, allow registration using an already registered domain
* Remove unused libraries folder
* Correctly handle order creation based on WooCommerce settings, improve admin notices (add missing dismiss buttons and additional support information for errors)
* Prevent double click on order synchronize button
* Handle refresh token expired error
* Do not round prices before sending them to WooCommerce Bexio Automator Service


## 1.4.0 (2018-05-31)

* Add order note as Bexio comment via remote API
* Add custom plugin links in plugin table
* Add support button and improve documentation
* Fix live stock synchronization
* Fix parent product requesting
* Restore compatibility with WooCommerce 2+


## 1.3.0 (2018-05-17)

* Fix stock synchronization problem if no products match in Bexio
* Implement file based stock cache to decrease stock requests
* Restrict company logo width to avoid breaking the visual design
* Fix wrongly formatted date of last product stock synchronization
* Add correct PHP opening tag to main class to avoid warnings
* Update cached options upon setting new options to fix non-working redirect after registration
* Handle products with disabled stock management correctly
* Update translations for new status messages
* Add new settings class with new system status information
* Add new functions for reading WooCommerce version and plugin update status to improve compatibility
* Refactor settings - extract to separate class for better maintainability
* Switch settings to meta box layout and add company information
* Add new function for requesting company information
* Change template architecture using index template
* Add support for WooCommerce fees
* Add explicit compatibility with WooCommerce via meta tags


## 1.2.0 (2018-02-06)

* Update plugin homepage
* Update translations and add Swiss German language files
* Remove no longer needed assets directory
* Add missing shipping account
* Add description, changelog, banner and logo to plugin update screen
* Fix plugin slug constant
* Improve update functionality and plugin information fetching
* Add new configuration parameter for the plugin slug
* Add auto-update handler
* Add new method for requesting latest plugin version
* Make error messages from API translatable
* Update translations for new authorization reset messages
* Implement new option to reset authorization data
* Recreate Swiss German translation file after merge
* Add new message translations
* Display delivery status if available
* Store delivery status upon response if available
* Pull all update information from the service API
* Add new configuration options
* Add new function for requesting Bexio contact groups


## 1.1.0 (2017-11-06)

* Remove unnecessary context for retrieving the plugin version
* Improve reading of the plugin version that also works in frontend
* Remove unnecessary product meta updates, already handled by WC
* Fix excessive logging of empty responses from the remote API
* Update stock info on stock update according to WC settings
* Use correct option for issue delivery
* Change stock request from GET to POST to avoid exceeding the URL length
* When Bexio is set to include tax into price, there was a calculation problem, which is now fixed
* Add new MDM action hooks for updating the product stock
* Return empty array rather than `null` to avoid a resulting error
* Only update the stock of products where the cache time is expired
* Add new todo for filtering Bexio accounts
* Update version for next release tag
* Make footer imprint italic
* Fix problem with non-saveable boolean options
* Remove redundant reset submit button
* Add new handlers for reseting plugin data and displaying the footer
* Add new function to reset all WooCommerce Bexio Automator data
* Fix fatal error when determining the current plugin version
* Add new translations for new setting views
* Add new template for reseting all user data
* Implement footer in the settings page with link to the Bexio shop site
* Make domain registration field readonly
* Add German translations for the newly introduced options
* Communicate immediate order issuing to the remote API
* Add new option for immediate order issuing
* Add new option for disabling immediate delivery creation
* Add option for brutto or netto prices to submit to Bexio
* Add util class to check version of WooCommerce
* General cleanup: remove unneeded code, improve doc, implement SRP and remove error logs
* Add external link to the Bexio order in the WooCommerce order table
* Change API to singleton to allow global error messages
* Fix problem with the product search and implement SRP
* Fix wrong method description
* Update translations for new column header
* Add new column in product list to shown last stock synchronization date
* Add filter for order delivery creation
* Fix spelling mistake
* Update translations with new placeholder descriptions
* Improve placeholder list, which is now collapsible
* Allow shipping method placeholder in the order title
* Log errors to php error log
* Fix options remove, this was renamed from `unset` to `remove`
* Fix get product from order item
* Allow Shop Managers to save Bexio settings
* Fix wrong settings URL after page was moved - changed to dynamically fetching the settings URL.
* Fix query data handling for GET requests
* Fix broken authorization URL in for OAuth2 authentication via Bexio
* Rename plugin to meet Raptus naming convention
* Remove cache time upon saving of a product to enforce a stock update in the backend
* Implement WooCommerce 2+ compatibility
* Change WooCommerce Bexio Automator Service URL to actual LIVE URL with possibility to override via global constants
* Move settings menu to a sub page of the WooCommerce settings
* Update translations base and add German translation
* Reformat code for compliance with Raptus coding guidelines
* Update to new options factory and add improve code documentation
* Add new template if client status is blocked
* Extracted option handling into separate factory class
* Update readme with installation instructions
* Ignore booking thank you logic when order is already set (refresh thank you page)
* Add order update and stock class
* Client registration and API class created
* Adapt readme for autoloading
* Switch to autoload rather than static class loading
* Fix typos in the readme file
* Add composer file
* Add template translation file
* Update plugin name
* Add initial file and folder structure
