<?php

// Namespace declaration
namespace Raptus\WC\BexioConnector;

/**
 * A project specific autoloader implementation that supports type file prefixes.
 *
 * After registering this autoload function with SPL, the following line
 * would cause the function to attempt to load the \Raptus\BexioConnector\Some\Class 
 * class from /path/to/project/classes/some/class.php
 *
 *      new \Raptus\WC\BexioConnector\Some\Class();
 *
 * @param string $class The fully-qualified class name.
 * @return void
 */
spl_autoload_register(function($class) {

    // Project-specific namespace prefix
    $prefix = __NAMESPACE__ . '\\';

    // Base directory for the namespace prefix
    $base_dir = defined($prefix . 'CLASSES_DIR') ? CLASSES_DIR : __DIR__ . '/classes/';

    // Check if the class uses our namespace prefix, if not,
    // abort and let the the next registered autoloader handle
    $len = strlen($prefix);
    if (strncmp($prefix, $class, $len) !== 0) {
        return;
    }

    // Get the relative class name
    $relative_class = substr($class, $len);

    // Replace the namespace prefix with the base directory, replace namespace separators with 
    // directory separators in the relative class name and append the .php file extension
    $path_parts = explode('/', $base_dir . strtolower(str_replace('\\', '/', $relative_class)) . '.php');

    // Extract isolated file and path names
    $file = array_pop($path_parts);
    $path = join('/', $path_parts);

    // Find the file, prefixes for classes, interfaces and traits are supported.
    $supported_file_names = array($file, "class.$file", "interface.$file", "trait.$file");
    foreach ($supported_file_names as $file_name) {
        if (file_exists("$path/$file_name")) {
            require "$path/$file_name";
            return;
        }
    }
});

?>