<?php

// Namespace declaration
namespace Raptus\WC\BexioConnector;

// Exit if accessed directly
defined('ABSPATH') or die();

/**
 * The main class/core module of this plugin.
 */
class Core {

    /**
     * Use Singleton trait to disallow multiple instances of this class.
     */
    use Singleton;

    /**
     * Constructs a new instance of this class.
     * Registers all required filter and action hooks.
     */
    protected function __construct() {
        add_action('plugins_loaded', array($this, 'i18n'));
        add_action('plugins_loaded', array($this, 'maybe_init'), 100);
    }

    /**
     * Maybe initialize all modules of this plugin.
     * If WooCommerce is not available, print an error message.
     */
    public function maybe_init() {
        if (!Utils::is_woocommerce_active()) {
            add_action('admin_notices', array($this, 'woocommerce_required'), 50);
            return;
        }
        $this->init();
        add_action('after_setup_theme', array($this, 'init_again'), 100);
    }

    /**
     * Initialize all necessary components of this plugin.
     */
    public function init() {
        if (is_admin()) {
            // Only backend
            Admin::instance();
            Settings::instance();
            Wizard::instance();
            Update::instance();
            Migrations::instance();
        } else {
            // Only frontend
            Stock::instance();
            Order::instance();
        }

        // Frontend and backend
        StatusMonitor::instance();
        Modules::load();
        Compatibility::load();
    }

    /**
     * Initialize again at a later hook, to allow the theme to directly affect hooks from the plugin.
     * */
    public function init_again() {
        Tasks::load();
    }

    /**
     * Registers this plugin for internationalization and loads the text domain translations.
     */
    public function i18n() {
        load_plugin_textdomain('raptus-wc-bexio-connector', false, basename(PLUGIN_DIR) . '/languages');
    }

    /**
     * Print the error message, that WooCommerce is required for the plugin to run.
     */
    public function woocommerce_required() {
        Utils::print_notice('warning', __('WooCommerce Bexio Automator requires WooCommerce to be activated', 'raptus-wc-bexio-connector'));
    }
}