<?php

namespace Raptus\WC\BexioConnector;

defined('ABSPATH') or die;

/**
 * Initialize the activated modules.
 */
class Modules {

    /**
     * The modules list.
     * @var array
     */
    static $modules;

    /**
     * Load the active modules.
     */
    public static function load($id = false) {
        foreach (self::get() as $module) {
            if ($id && $module->get_id() != $id) {
                continue;
            }
            if($module->active())
                $module->activate();
            else
                $module->deactivate();
        }
    }

    /**
     * Get a list of all active modules.
     *
     * @return array The active modules.
     */
    public static function get() {
        if (!static::$modules) {
            /**
             * Filter the WooCommerce Bexio Automator modules. Useful to add a custom extension.
             * Modules must inherit from the class `\Raptus\WC\BexioConnector\Modules\Module`.
             *
             * @param array $moduleClasses The module classes
             */
            $moduleClasses = apply_filters('raptus_wc_bexio_connector_modules', [
                \Raptus\WC\BexioConnector\Modules\ProductAccounts::class,
                \Raptus\WC\BexioConnector\Modules\StockSync::class,
                \Raptus\WC\BexioConnector\Modules\PDFAttachments::class,
                \Raptus\WC\BexioConnector\Modules\DocumentNumbers::class,
                \Raptus\WC\BexioConnector\Modules\ProductImport::class,
                \Raptus\WC\BexioConnector\Modules\CountrySettings::class,
                \Raptus\WC\BexioConnector\Modules\ExtendedStateSync::class,
                \Raptus\WC\BexioConnector\Modules\DiscountSync::class
            ]);

            foreach ($moduleClasses as $moduleClass) {
                $module = new $moduleClass();
                static::$modules[$module->get_id()] = $module;
            }
        }

        return static::$modules;
    }

    /**
     * Pivate constructor to avoid instances of this factory class.
     */
    private function __construct() {}
}
