<?php

// Namespace declaration
namespace Raptus\WC\BexioConnector;

// Exit if accessed directly
defined('ABSPATH') or die();

/**
 * Monitor all order status transitions.
 */
class StatusMonitor {

    /**
     * Use Singleton trait to disallow multiple instances of this class.
     */
    use Singleton;

    /**
     * Constructs a new instance of this class.
     * Registers all required filter and action hooks.
     * Starts the status monitor.
     */
    protected function __construct() {
        if ($this->active()) {
            add_action('init', [$this, 'monitor']);
        }
    }

    /**
     * Monitor all order status transitions.
     * Important is also to catch the initial order status (not actually a transistion).
     */
    public function monitor() {
        // Trigger sync for orders created via checkout
        add_action('woocommerce_checkout_update_order_meta', [Order::instance(), 'sync_all'], 10, 1);

        // Trigger sync on status transitions
        foreach (Data::get_order_states() as $status => $name) {
            add_action('woocommerce_order_status_' . $status, [Order::instance(), 'sync_all'], 10, 1);
        }
    }

    /**
     * Test if the status monitor is active.
     */
    protected function active() {
        return Options::get('bexio_async_order_creation') != 'async';
    }
}