<?php

namespace Raptus\WC\BexioConnector;

defined('ABSPATH') or die;

/**
 * Initialize the activated tasks.
 */
class Tasks {

    /**
     * The tasks list.
     * @var array
     */
    static $tasks;

    /**
     * Load all registered tasks.
     */
    public static function load() {
        foreach (self::get() as $task) {
            $task->activate();
        }

        add_filter('cron_schedules', '\Raptus\WC\BexioConnector\Tasks::cron_schedules');
    }

    /**
     * Add custom wp-cron schedules.
     *
     * @return array The available wp-cron schedules.
     * */
    public static function cron_schedules($schedules) {
        $schedules['5min'] = [
            'interval' => 300,
            'display' => _x('Once every 5 minutes', 'Custom cron schedule display name', 'raptus-wc-bexio-connector')
        ];
        return $schedules;
    }

    /**
     * Unload all registered tasks.
     */
    public static function unload() {
        foreach (self::get() as $task) {
            $task->deactivate();
        }
    }

    /**
     * Get a list of all active tasks.
     *
     * @return array The active tasks.
     */
    public static function get($id = null) {
        if (!static::$tasks) {

            /**
             * Filter the WooCommerce Bexio Automator tasks. Useful to add a custom tasks.
             * Tasks must inherit from the class `\Raptus\WC\BexioConnector\Tasks\Task`.
             *
             * @param array $taskClasses The task classes
             */
            $taskClasses = apply_filters('raptus_wc_bexio_connector_tasks', [
                \Raptus\WC\BexioConnector\Tasks\AsyncOrders::class,
                \Raptus\WC\BexioConnector\Tasks\AsyncNewOrders::class,
                \Raptus\WC\BexioConnector\Tasks\SyncOrderStates::class,
            ]);

            foreach ($taskClasses as $taskClass) {
                $task = new $taskClass();
                static::$tasks[$task->get_id()] = $task;
            }
        }

        if ($id && isset(static::$tasks[$id])) {
            return static::$tasks[$id];
        }

        return static::$tasks;
    }

    /**
     * Pivate constructor to avoid instances of this factory class.
     */
    private function __construct() {}
}
