<?php

// Namespace declaration
namespace Raptus\WC\BexioConnector\Tasks;

// Exit if accessed directly
defined('ABSPATH') or die();

use Raptus\WC\BexioConnector\Options;
use Raptus\WC\BexioConnector\Modules;
use Raptus\WC\BexioConnector\Modules\ProductImport;

/**
 * Synchronize Bexio order states back to WooCommerce.
 */
class AsyncPrice extends Task {

    /**
     * Creates a new cron task to synchronize order states hourly.
     * If the order state synchronization is enabled, the task schedule is setup.
     * If the order state synchronization is disabled, all prevously registered tasks are cleared.
     */
    public function __construct() {
        parent::__construct(
            'raptus_sync_bexio_product_prices',
            apply_filters('raptus_wc_bexio_connector_sync_price_recurrence', 'twicedaily'),
            apply_filters('raptus_wc_bexio_connector_sync_price_interval', 12 * HOUR_IN_SECONDS)
        );
    }

    /**
     * Test if this task is active.
     */
    public function active() {
        return Options::get('bexio_sync_product_price', false);
    }

    /**
     * Execute the cron task.
     */
    public function execute() {
        Modules::get()['productimport']->sync_product_price();
    }
}
