<?php

// Namespace declaration
namespace Raptus\WC\BexioConnector\Tasks;

// Exit if accessed directly
defined('ABSPATH') or die();

use Raptus\WC\BexioConnector\Options;
use Raptus\WC\BexioConnector\Stock;

/**
 * Synchronize WooCommerce stock asynchronously with Bexio.
 */
class AsyncStock extends Task {

    /**
     * Creates a new cron task to synchronize product stock hourly.
     * If the asynchronous synchronization is enabled, the task schedule is set up.
     * If it is set to regular, all prevously registered tasks are cleared.
     */
    public function __construct() {
        parent::__construct('raptus_sync_bexio_stock');
    }

    /**
     * Test if this task is active.
     */
    public function active() {
        return Options::get('bexio_async_stock_sync') == 'async';
    }

    /**
     * Exectute the cron task.
     */
    public function execute() {
        $this->sync_stock();
    }

    /**
     * Transition all WooCommerce order states to the corresponding state
     * of the associated remote Bexio order.
     */
    protected function sync_stock() {
        // Force read update from bexio
        Stock::instance()->maybe_update_stock_cache(true);
        foreach (Stock::instance()->iter_stock() as $sku => $row) {
            $product_id = wc_get_product_id_by_sku($sku);
            $product = wc_get_product($product_id);
            if (!$product->managing_stock() || empty($product->get_sku())) {
                continue;
            }
            if ($product) {
                Stock::instance()->update_stock($row->stock, $product);
            }
        }
    }
}
