# WooCommerce Bexio Automator Wordpress Plugin

This plugin provides a connection between WooCommerce and Bexio using the WooCommerce Bexio Automator Service.

## Features
The following features are currently implemented:

- Synchronize stock status of products with matching product numbers
- Synchronize entire orders including fees, discounts, contacts and addresses
- Synchronize the status of orders created by the plugin
- Create and issue deliveries based on payment gateway and order status
- Create, issue and pay invoices based on payment gateway and order status
- Set Bexio accounts on products and product categories
- Connect WooCommerce payment methods with Bexio bank accounts
- Upgrade, downgrade or cancel your subscription from within the plugin settings
- Manually synchronize the stock of products and individual orders

## Installation
- Download, install and activate this plugin
- Navigate to *WooCommerce > WooCommerce Bexio Automator* and enter the customer data to register a new client
 - The domain should be the current domain where the plugin is installed, this is the default value
- Save the data to perform a registration
 - Your trial period starts now
 - If the registration is successful you'll see a button to grant permission
- Click the button and you'll be redirected to Bexio
 - Login and select a your Bexio company
 - Press the grant permission button on the next page
 - You'll be redirect back to the WordPress backend
- On success, you'll see a new options section where you can configure synchronization settings
 - The selectable options come directly from Bexio

## Support
- Consult our information section on the [WooCommerce Bexio Automator Website](https://bexio-shop.ch/) for [instructions](https://bexio-shop.ch/inbetriebnahme/), terms and conditions and [FAQ](https://bexio-shop.ch/faq/)
- If you need help with the configuration of the synchronization rules, please visit our dedicated [synchronization rule information page](https://helpcenter.bexio-shop.ch/hc/de/articles/9606928850961)
- For all other questions please visit our [service desk support platform](https://bexio-shop.ch/support/)

<br>

# WooCommerce Bexio Automator Hooks Reference

## Chapters

- General Filters
- Filters in Modules
- Filters in Tasks (Cron-Jobs)
- Actions

<br>

# General Filters

## WooCommerce Orders in Admin Dashboard Filters

### classes/class.admin.php:

#### 1. raptus_wp_bexio_connector_edit_link_' . $type

`apply_filters('raptus_wp_bexio_connector_edit_link_' . $type, $html, $order_id)`

Filters the **html of the links to Bexio** in the WooCommerce overview of orders. Default value for $type is 'order'. Other values
are 'delivery' and 'invoice'.

#### Parameters

| Identifier | Type | Description |
| ---------- | -----| ----------- |
| $html | *string* | The html of the link to edit |
| $order_id | *int* | The id of WC order |

#### 2. raptus-wc-bexio-connector-stock-status

`apply_filters('raptus-wc-bexio-connector-stock-status', $status, $post_id)`

Filters the **stock status timestamp** displayed in the WooCommerce overview of products.

#### Parameters

| Identifier | Type | Description |
| ---------- | -----| ----------- |
| $status | *string* | timestamp of last synchronization |
| $post_id| *int* | The id of the product |


## Remote Logging and API Route Filters

### classes/class.api.php:

#### 1. raptus_wc_bexio_connector_remote_log

`apply_filters('raptus_wc_bexio_connector_remote_log', true)`

Filter to deactivate remote logging to **Google Gloud Logging**. Set to false to deactivate remote logging. Note: this only disables remote logging _from within the plugin itself_ - it will not affect remote logging for occurences in the plugin service. If you want to completely disable remote logging, contact the support and state your wish to disable logging consent (this might degrade the quality of support in certain cases).

#### Parameters

| Identifier | Type | Description |
| ---------- | -----| ----------- |
| - | *bool* | activate/deactivate remote logging, default is true |

#### 2. raptus_wc_bexio_connector_request_route

`apply_filters('raptus_wc_bexio_connector_request_route', $route, $method, $data)`

Allows filtering the **basic request route** to the Plugin service. 

#### Parameters

| Identifier | Type | Description |
| ---------- | -----| ----------- |
| $route | *string* | relative request route to send the request to |
| $method | *string* | request method, i.e. 'GET' |
| $data | *array* | the request data array to send |

## Default Settings Filter

### classes/class.defaults.php:

#### 1. raptus_wc_bexio_connector_defaults

`apply_filters('raptus_wc_bexio_connector_defaults', static::$defaults)`

Filters the **default values** during initial setup.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $defaults  | *array* | defaults for the plugin settings |

## Order Synchronization Filters

### classes/class.order.php:

#### 1. raptus_wc_bexio_connector_invoice_data

`apply_filters('raptus_wc_bexio_connector_invoice_data', $data, $order)`

Allows filtering the **invoice data** before an invoice is created.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $data | *array* | the information needed to create a invoice |
| $order | *WC_Order* | the WooCommerce order |

#### 2. raptus_wc_bexio_connector_delivery_data

`apply_filters('raptus_wc_bexio_connector_delivery_data', $data, $order)`

Allows filtering the **delivery data** before a delivery is created.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $data | *array* | the information needed to create an delivery |
| $order | *WC_Order* | the given WooCommerce Order |

#### 3. raptus_wc_bexio_connector_sync_and_deliver

`apply_filters('raptus_wc_bexio_connector_sync_and_deliver', $bexio_order_id && $order->is_paid() && get_post_meta($order_id, '_bexio_delivery_issued', true) != 1, $order)`

Allows to change the condition if a **delivery** should be issued for for a given WooCommerce order.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| -  | *bool* | true if delivery should be issued, false if not |
| $order | *WC_Order* | the given WooCommerce Order |

#### 4. raptus_wc_bexio_connector_order_data

`apply_filters('raptus_wc_bexio_connector_order_data', $order_data, $order)`

Filters the **order data** of a WooCommerce order before it is being synchronized with Bexio.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $order_data | *array* | the collected WooCommerce order data |
| $order | *WC_Order* | the given WooCommerce Order |

#### 5. raptus_wc_bexio_connector_fee_data

```
apply_filters('raptus_wc_bexio_connector_fee_data', array(
    'name' => $fee['name'],
    'price' => $this->round_price($amount),
    'tax_id' => Options::get('bexio_tax_' . ($fee['tax_class'] ?: 'standard')),
    'account_id' => Options::get('bexio_account_products')
), $order)
```

Filters the **fees** array of a given WooCommerce order

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *array* | the fee data consisting of name, price, tax_id and account_id |
| $order | *WC_Order* | the given WooCommerce Order |

#### 6. raptus_wc_bexio_connector_shipping_data

```
apply_filters('raptus_wc_bexio_connector_shipping_data', array(
    'name' => apply_filters('raptus_wc_bexio_connector_shipping_title', $order->get_shipping_method(), $order),
    'price' => $this->round_price($shipping),
    'tax_id' => Options::get('bexio_tax_shipping'),
    'account_id' => Options::get('bexio_account_shipping')
), $order)
```

Filter the **shipping data** of a given WooCommerce order

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *array* | the shipping data consisting of name, price, tax_id and account_id |
| $order | *WC_Order* | the given WooCommerce order |

#### 7. raptus_wc_bexio_connector_shipping_title

`apply_filters('raptus_wc_bexio_connector_shipping_title', $order->get_shipping_method(), $order)`

Filter the **title of a shipping method** for a given WooCommerce order.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *string* | title of the shipping method |
| $order | *WC_Order* | the given WooCommerce order|

#### 8. 'raptus_wc_bexio_connector_order_title

`apply_filters('raptus_wc_bexio_connector_order_title', $title, $order)`

Filters the **order title** of a given WooCommerce order.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $title | *string* | the title of the order |
| $order | *WC_Order* | the given WooCommerce order |

#### 9. raptus_wc_bexio_connector_order_title_template_replacements

```
apply_filters('raptus_wc_bexio_connector_order_title_template_replacements', array(
    'order_id'             => $order_id,
    'order_no'             => $order->get_order_number(),
    'shipping_method'      => $order->get_shipping_method(),
    'shipping'             => $this->get_customer_address($order, 'shipping'),
    'billing'              => $this->get_customer_address($order, 'billing'),
    'currency'             => $this->get_currency($order),
    'payment_method'       => $this->get_payment_method($order),
    'payment_method_title' => $this->get_payment_method_title($order),
    'top_product'          => $this->get_top_product($order),
), $order_id, $order)
```

Filters the **order title template replacements**.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *array* | template replacements for order title |
| $order_id | *int* | the id of the order |
| $order | *WC_Order* | the given WooCommerce order |


## Settings Filters

### classes/class.settings.php:

#### 1. raptus_wc_bexio_connector_settings_base_template

`apply_filters('raptus_wc_bexio_connector_settings_base_template', TEMPLATES_DIR . 'index.php')` 

Filters the path to the **base template of the settings** of plugin

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| -  | *string* | path to the base template|

#### 2. raptus_wc_bexio_connector_settings_template_args

`apply_filters('raptus_wc_bexio_connector_settings_template_args', [])`

Filters the **arguments for the template**. Default is empty array.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *array* | arguments for the template |

#### 3. raptus_wc_bexio_connector_settings_template

`apply_filters('raptus_wc_bexio_connector_settings_template', "$template.php")`

Filters the **template variable**.

#### Pararmeters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *string* | name of template file |

#### 4. raptus_wp_bexio_connector_register_setting/'.$id

`apply_filters('raptus_wp_bexio_connector_register_setting/'.$id, true, $page, $section)`

Allows prevention of **registration of a settings field** via filter.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
|- | *bool* | whether to register a certain setting or not |
| $page | *string* | the relevant page |
| $section | *string* |the relevant section |

#### 5. raptus_wc_bexio_connector_sanitize_settings

`apply_filters('raptus_wc_bexio_connector_sanitize_settings', $new_input, $input)`

Filters the $new_input array after the $input array has been sanitized.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $new_input  | *array* | the sanitized settings fields |
| $input | *array* | the settings fields beforehand |

#### 6. raptus_wc_bexio_connector_order_title_template_placeholders

`apply_filters('raptus_wc_bexio_connector_order_title_template_placeholders', $placeholders)`

Filters the placeholders for the Bexio order title pattern.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $placeholders | *array* | the input fields placeholders |


## Stock Synchronization Filters

### classes/class.stock.php:


#### 1. raptus_wc_bexio_connector_update_stock

```
apply_filters('raptus_wc_bexio_connector_update_stock',
    $amount > 0 ? 'instock' : ($product->backorders_allowed() ? 'onbackorder' : 'outofstock'),
    $product,
    $amount
)
```

Filters the **stock status** when stock of a given product is updated.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| stock status | *string* | the stock status to be set for the product |
| $product | *WC_Product* | the given WC product |
| $amount |*array int* | the stock amount of the product |

#### 2. raptus_wc_bexio_connector_stock_fetch_mode

`apply_filters('raptus_wc_bexio_connector_stock_fetch_mode', null)`

Possiblity to change the **fetch mode** when **stock** is fetched from Bexio. Possible STRING values: available, effective, reserved, picked, unpicked.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | * | string* | fetch stock based on fetch mode |

#### 3. raptus_wc_bexio_connector_stock_products

`apply_filters('raptus_wc_bexio_connector_stock_products', array(), array_filter($skus))`

Set stock **fetch mode** for individual sku's. Possiblity to change the **sku modes** (Array) before stock is fetched from Bexio.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *array* | sku modes | 
| -  | *array* | skus |


#### 5. raptus_wc_bexio_connector_enable_stock_synchronization

`apply_filters('raptus_wc_bexio_connector_enable_stock_synchronization', get_option('woocommerce_manage_stock') === 'yes' && !Options::get('bexio_disable_stock_synchronization', false))`

Filter to change if **stock synchronization is enabled**.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *bool*  | return true to enable stock synchronization, false otherwise |

#### 6. raptus_wc_bexio_connector_cache_table_varchar_size

`apply_filters('raptus_wc_bexio_connector_cache_table_varchar_size', 255)`

Filter to change the **size of the sku field** in the database table **rwcbc_stock** used for stock caching. Reduce the value if there are database errors related to this database table.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *int* | VARCHAR size |


## Sync Rules Filters

### /classes/class.syncrules.php:

#### 1. raptus-wc-bexio-connector-sync- . $type . \_gateway

`apply_filters('raptus-wc-bexio-connector-sync-' . $type . '_gateway', Order::instance()->get_payment_method($order), $order)`

Filter to change the **gateway id for the payment method** of the order for order, invoice, delivery or payment.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *int* | gateway id |
| $order | *WC_Order* | the given WooCommerce order |


#### 2. raptus-wc-bexio-connector-sync- . $type

`apply_filters('raptus-wc-bexio-connector-sync-' . $type, $can_sync, $order)`

Filter the **can_sync** variable for a specific order for Bexio order, invoice, delivery or payment.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $can_sync | *bool* |  whether a given order should be synced or not |
| $order | *WC_Order* | the given WooCommerce order |

#### 3. raptus_wc_bexio_connector_order_state_priorities

```
apply_filters('raptus_wc_bexio_connector_order_state_priorities', array(
    'pending' => 10,
    'on-hold' => 20,
    'processing' => 30,
    'completed' => 40,
    'refunded' => 50,
))
```

Filter to change the array of **order states**.      

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
|-  | *array* | order states |

the mapping of WooCommerce to Bexio order states

<br>       

# Filters in Modules (premium)

## Module Registration Filter

### classes/class.modules.php:

#### 1. raptus_wc_bexio_connector_modules

```
apply_filters('raptus_wc_bexio_connector_modules', [
    \Raptus\WC\BexioConnector\Modules\ProductAccounts::class,
    \Raptus\WC\BexioConnector\Modules\StockSync::class,
    \Raptus\WC\BexioConnector\Modules\PDFAttachments::class,
    \Raptus\WC\BexioConnector\Modules\DocumentNumbers::class,
    \Raptus\WC\BexioConnector\Modules\ProductImport::class,
    \Raptus\WC\BexioConnector\Modules\CountrySettings::class,
    \Raptus\WC\BexioConnector\Modules\ExtendedStateSync::class,
    \Raptus\WC\BexioConnector\Modules\DiscountSync::class
]);
```

Filter which enables adding a **custom module** to the module loading mechanism.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
|-  | *array* | module classes |


## Module Document Numbers Filter

### classes/modules/class.documentnumbers.php:

##### 1. raptus_wc_bexio_connector_document_number

`apply_filters('raptus_wc_bexio_connector_document_number', $documentNumber, $document, $order)`

Filter which allows modification of the **document number**.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $documentNumber | *string* | the document number to be modified |
| $document | *string* | the document |
| $order | *WC_Order* | the given WooCommerce order |

## Module Extended State Sync Filter

### classes/modules/class.extendedstatesync.php:

#### 1. raptus-wc-bexio-connector-deactivate-sync-payment-rules

`apply_filters('raptus-wc-bexio-connector-deactivate-sync-payment-rules', true)` 

Filter to activate (false) or deactivate (true) the **synchronisation of payments**.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| -  | *bool* | activate/deactivate sync of payments |



## Module PDF Attachment Filters

### classes/modules/class.pdfattachments.php:

#### 1. raptus_wp_bexio_connector_invoice_pdf_upload_directory

`apply_filters('raptus_wp_bexio_connector_invoice_pdf_upload_directory', '/invoices/')`

Filter to change the folder in uploads where **PDF**'s are stored.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *string* | folder name for PDF's |

#### 2. raptus_wp_bexio_connector_invoice_pdf_file_name

`apply_filters('raptus_wp_bexio_connector_invoice_pdf_file_name', $pdf['name'], $dest, $pdf)`

Filter to change the **PDF filename**.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| -  | *string* | the pdf file name |
| $dest | *string* | the file destination path |
| $pdf | *array* | the given pdf |

#### 3. raptus_wp_bexio_connector_can_send_invoice

`apply_filters('raptus_wp_bexio_connector_can_send_invoice', $can_send, $type, $order)`

Filter to decide if an **invoice** can be sent based on the configuration rules.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $can_send | *bool* | true if the invoice can be sent, false otherwise |
| $type | *string* | email type |
| $order | *WC_Order* | the given WooCommerce order |


## Module Product Import Filter

### classes/modules/class.productimport.php:

#### 1. raptus_wc_bexio_connector_import_product

`apply_filters('raptus_wc_bexio_connector_import_product', true, $product, $wc_product)`

Filter to exclude a product from **product import**.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *bool* | whether to import a product or not |
| $product | *array* | the Bexio product. Array keys: name, description, sku, sale_price, width, height, weight, stock_nr, stock_reserved_nr, stock_picked_nr, stock_min_nr |
| $wc_product | *WC_Product* | the corresponding WooCommerce Product |


#### 2. raptus_wc_bexio_connector_import_product_before_save

`apply_filters('raptus_wc_bexio_connector_import_product_before_save', $wc_product, $product)`

Filter to change a product before it is being updated and [saved](https://woocommerce.github.io/code-reference/classes/WC-Product.html#method_save) in WooCommerce.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $wc_product | *WC_Product* | the WooCommerce product to be updated |
| $product | *array* | the Bexio product to be imported. Array keys: name, description, sku, sale_price, width, height, weight, stock_nr, stock_reserved_nr, stock_picked_nr, stock_min_nr |


#### 3. raptus_wc_bexio_connector_create_product_before_save

`apply_filters('raptus_wc_bexio_connector_create_product_before_save', $wc_product, $product)`

Filter to change a product before it is being created and [saved](https://woocommerce.github.io/code-reference/classes/WC-Product.html#method_save) in WooCommerce.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $wc_product | *WC_Product* | the WooCommerce product to be saved |
| $product | *array* | the Bexio product to be imported |

<br>

# Filters in Tasks (Cron-Jobs)

## Task Registration Filter
 
### classes/class.tasks.php:

#### 1. raptus_wc_bexio_connector_tasks

```
apply_filters('raptus_wc_bexio_connector_tasks', [
    \Raptus\WC\BexioConnector\Tasks\AsyncOrders::class,
    \Raptus\WC\BexioConnector\Tasks\AsyncNewOrders::class,
    \Raptus\WC\BexioConnector\Tasks\SyncOrderStates::class,
])
```

Filter to add **task classes** to the static tasks array.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *array* | the task classes |

## Task asynchronous price synchronization Filter

### classes/tasks/class.asyncprice.php:

#### 1. raptus_wc_bexio_connector_sync_price_recurrence

`apply_filters('raptus_wc_bexio_connector_sync_price_recurrence', 'twicedaily')`

Filter to set **recurrence name of async price interval**.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *string* | recurrence |

#### 2. raptus_wc_bexio_connector_sync_price_interval 

`apply_filters('raptus_wc_bexio_connector_sync_price_interval', 12 * HOUR_IN_SECONDS)` 

Filter to set **interval for next async price repeat**.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *int* | interval |

## Task for invoice state synchronization Filter

### classes/tasks/class.syncinvoicestate.php:

#### 1. raptus_wc_bexio_connector_transition_invoice_state

`apply_filters('raptus_wc_bexio_connector_transition_invoice_state', true, $order)`

Filter to skip **sync of invoice state** for a specific order (return `false` to skip).

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *bool* | whether invoice state from bexio should be transitioned to the WooCommerce order |
| $order | *WC_order* | the given WooCommerce order |


## Task for order state synchronization Filter

### classes/tasks/class.syncorderstates.php:

#### 2. raptus_wc_bexio_connector_transition_order_state

`apply_filters('raptus_wc_bexio_connector_transition_order_state', true, $order)`

Filter to skip **sync of order state** for a specific order (return `false` to skip).

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *bool* | whether order state in Bexio should be transitioned to the WooCommerce order |
| $order | *WC_order* | the given WooCommerce order |


## Task for asynchronous synchronization of orders to Bexio Filter

### classes/tasks/class.asyncneworders.php:

#### 1. raptus_wc_bexio_connector_seperate_increased_interval_sync_new_orders

`apply_filters('raptus_wc_bexio_connector_seperate_increased_interval_sync_new_orders', Options::get('bexio_async_order_creation') == 'async')`

Filter to disable seperate **sync of _new_ orders** executed _every 5 minutes_ (use `__return_false` to disable).

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *bool* | whether _new_ orders (orders with no data synced yet) should be synced seperately, every 5 minutes (independent of the 1x / hour sync) |

<br>

# Actions

## Actions for Plugin Settings

### classes/class.settings.php

#### Meta boxes

#### 1. raptus_wc_bexio_connector_meta_boxes

`do_action('raptus_wc_bexio_connector_meta_boxes', $this->page_hook)`

Action which hooks into the **meta boxes**. Fires after the metaboxes were added.

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| - | *string* | page hook |

#### Options Page

#### 1. raptus_wc_bexio_connector_render_options_page

`do_action('raptus_wc_bexio_connector_render_options_page')`

Action which fires before the rendering of the **options page**.

#### Parameters

- none

#### Additional Settings

#### 1. raptus_wp_bexio_connector_after_settings/$id

`do_action('raptus_wp_bexio_connector_after_settings/'.$id, $page, $section)`

Allows adding **additional settings** after a specific setting via action. Example $id: "bexio_disable_taxes".

#### Parameters

| Identifier | Type | Description |
| ---------- | ---- | ----------- |
| $page | *string* | the settings page |
| $section | *string* | the section of the settings |
